<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\ClearanceDocumentUsa;
use Modules\Sd\Entities\ClearanceDocumentUsaAttachment;
use Modules\Sd\Entities\Shipment;
use Modules\Sd\Entities\ShippingAgent;

class ClearanceDocumentUsaController extends Controller
{
    public function getCDUsaAllData()
    {
        try {
            $query = 'SELECT * FROM sd_clearance_documents_usa ORDER BY `created_at` DESC';
            $clearance_doc_usa_datas = DB::select($query);
            $result = array();

            foreach ($clearance_doc_usa_datas as $clearance_doc_usa_data) {
                array_push($result, [
                    "customer_order_id" => $clearance_doc_usa_data->customer_order_id,
                    "factory_po" => $clearance_doc_usa_data->factory_po,
                    "shipping_agent_id" => $clearance_doc_usa_data->shipping_agent_id,
                    "shipment_id" => $clearance_doc_usa_data->shipment_id,
                    "cd_usa_id" => $clearance_doc_usa_data->cd_usa_id,
                ]);
            }
            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "getCDUsaAllData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "getCDUsaAllData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function saveCdUsa(Request $request)
    {
        try {
            $cd_usa = new ClearanceDocumentUsa();
            $cd_usa->customer_order_id = $request->get('customer_order_id');
            $cd_usa->factory_po = $request->get('factory_po');
            $cd_usa->shipping_agent_id = $request->get('shipping_agent_id');
            $cd_usa->shipment_id = $request->get('shipment_id');
            $cd_usa->note = $request->get('note');

            if ($cd_usa->save()) {
                // $this->sendEmail();
                $attachments = ClearanceDocumentUsaAttachment::where([['sd_cd_usa_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->cd_usa_id = $cd_usa->cd_usa_id;
                    $attachment->update();
                }
                $attachments = ClearanceDocumentUsaAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->cd_usa_id = $cd_usa->cd_usa_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "saveCdUsa", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "saveCdUsa", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function updateCdUsa(Request $request, $id)
    {
        try {
            $cd_usa = ClearanceDocumentUsa::find($id);
            $cd_usa->customer_order_id = $request->get('customer_order_id');
            $cd_usa->factory_po = $request->get('factory_po');
            $cd_usa->shipping_agent_id = $request->get('shipping_agent_id');
            $cd_usa->shipment_id = $request->get('shipment_id');
            $cd_usa->note = $request->get('note');
            if ($cd_usa->update()) {
                $attachments = ClearanceDocumentUsaAttachment::where([['cd_usa_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->cd_usa_id = $cd_usa->cd_usa_id;
                    $attachment->update();
                }
                $attachments = ClearanceDocumentUsaAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->cd_usa_id = $cd_usa->cd_usa_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "updateCdUsa", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "updateCdUsa", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteCdUsa(Request $request, $id)
    {
        try {
            $cd_usa = ClearanceDocumentUsa::where('cd_usa_id', '=', $id);
            $rec_data_attachments = ClearanceDocumentUsaAttachment::where('cd_usa_id', '=', $id)->get();

            if ($cd_usa->delete()) {
                foreach ($rec_data_attachments as $rec_data_attachment) {
                    $rec_data_attachment->delete();
                }
            }

            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "deleteCdUsa", "sucess");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "deleteCdUsa", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCdDocumentData($id)
    {
        try {
            $recordHead = ClearanceDocumentUsa::find($id);

            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "getCdDocumentData", $recordHead);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShipmentData()
    {
        try {
            $shipmet = Shipment::all();

            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "getShipmentData", $shipmet);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "getShipmentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShippingAgentData()
    {
        try {
            $shipping_agent = ShippingAgent::all();

            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "getShippingAgentData", $shipping_agent);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "getShippingAgentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCustomerOrderData($id)
    {
        try {
            $query = 'SELECT
            sd_shipment.customer_order_id,
            sd_shipment.shipment_id,
            pp_customer_orders.factory_po_num
        FROM
            sd_shipment
            LEFT JOIN pp_customer_orders ON sd_shipment.customer_order_id = pp_customer_orders.order_id
        WHERE
            sd_shipment.shipment_id = ' . $id;
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "customer_order_id" => $order->customer_order_id,
                    "factory_po_num" => $order->factory_po_num,
                    "shipment_id" => $order->shipment_id,
                ]);
            }


            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "getCustomerOrderData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function uploadScheduleAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = 'http://127.0.0.1:8000/clearance_documents_usa/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('clearance_documents_usa'), $filename);

        $cd_usa_attachment = new ClearanceDocumentUsaAttachment();
        $cd_usa_attachment->sd_cd_usa_attachment_id = 0;
        $cd_usa_attachment->description = $description;
        $cd_usa_attachment->file_path = $filename;
        $cd_usa_attachment->token = $token;
        return $cd_usa_attachment->save();
    }

    public function allScheduleAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT * FROM sd_clearance_documents_usa_attachments WHERE  cd_usa_id = "' . $id . '" OR cd_usa_id = "0"');
            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "allCdUsaAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "allCdUsaAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteScheduleAttachment($id)
    {
        try {
            $rec_data = ClearanceDocumentUsaAttachment::where('sd_cd_usa_attachment_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $schedule = $data;
                $data->delete();
            }
            $schedule = true;
            $responseBody = $this->responseBody(true, "ClearanceDocumentUsaController", "delete schedule attachment", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ClearanceDocumentUsaController", "delete schedule attachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
